@@ -31,12 +31,12 @@ from collections.abc import Sequence
 from pathlib import Path
 from random import randint
 from shutil import which
-from typing import TYPE_CHECKING, Union
+from typing import TYPE_CHECKING
 
 try:
-    from dbus_next import AuthError, Message, Variant
-    from dbus_next.aio import MessageBus
-    from dbus_next.constants import BusType, MessageType
+    from dbus_fast import AuthError, Message, Variant
+    from dbus_fast.aio import MessageBus
+    from dbus_fast.constants import BusType, MessageType
 
     has_dbus = True
 except ImportError:
@@ -44,10 +44,11 @@ except ImportError:
 
 from libqtile.log_utils import logger
 
-ColorType = Union[str, tuple[int, int, int], tuple[int, int, int, float]]
-ColorsType = Union[ColorType, list[ColorType]]
+ColorType = str | tuple[int, int, int] | tuple[int, int, int, float]
+ColorsType = ColorType | list[ColorType]
 if TYPE_CHECKING:
-    from typing import Any, Callable, Coroutine, TypeVar
+    from collections.abc import Callable, Coroutine
+    from typing import Any, TypeVar
 
     T = TypeVar("T")
 
@@ -111,7 +112,7 @@ def rgb(x: ColorType) -> tuple[float, float, float, float]:
 
     Which is returned as (1.0, 0.0, 0.0, 0.5).
     """
-    if isinstance(x, (tuple, list)):
+    if isinstance(x, tuple | list):
         if len(x) == 4:
             alpha = x[-1]
         else:
@@ -141,7 +142,7 @@ def rgb(x: ColorType) -> tuple[float, float, float, float]:
 
 def hex(x: ColorType) -> str:
     r, g, b, _ = rgb(x)
-    return "#%02x%02x%02x" % (int(r * 255), int(g * 255), int(b * 255))
+    return f"#{int(r * 255):02x}{int(g * 255):02x}{int(b * 255):02x}"
 
 
 def has_transparency(colour: ColorsType) -> bool:
@@ -151,7 +152,7 @@ def has_transparency(colour: ColorsType) -> bool:
     Where a list of colours is passed, returns True if any
     colour is not fully opaque.
     """
-    if isinstance(colour, (str, tuple)):
+    if isinstance(colour, str | tuple):
         return rgb(colour)[3] < 1
     return any(has_transparency(c) for c in colour)
 
@@ -160,7 +161,7 @@ def remove_transparency(colour: ColorsType):  # type: ignore
     """
     Returns a tuple of (r, g, b) with no alpha.
     """
-    if isinstance(colour, (str, tuple)):
+    if isinstance(colour, str | tuple):
         return tuple(x * 255.0 for x in rgb(colour)[:3])
     return [remove_transparency(c) for c in colour]
 
@@ -228,7 +229,7 @@ def describe_attributes(obj: Any, attrs: list[str], func: Callable = lambda x: x
     for attr in attrs:
         value = getattr(obj, attr, None)
         if func(value):
-            pairs.append("%s=%s" % (attr, value))
+            pairs.append(f"{attr}={value}")
 
     return ", ".join(pairs)
 
@@ -266,13 +267,13 @@ def lazify_imports(
     """
     __all__ = tuple(registry.keys())
 
-    def __dir__() -> tuple[str, ...]:
+    def __dir__() -> tuple[str, ...]:  # noqa: N807
         return __all__
 
-    def __getattr__(name: str) -> Any:
+    def __getattr__(name: str) -> Any:  # noqa: N807
         if name not in registry:
             raise AttributeError
-        module_path = "{}.{}".format(package, registry[name])
+        module_path = f"{package}.{registry[name]}"
         return import_class(module_path, name, fallback=fallback)
 
     return __all__, __dir__, __getattr__
@@ -294,7 +295,7 @@ def send_notification(
     https://developer.gnome.org/notification-spec/
     """
     if not has_dbus:
-        logger.warning("dbus-next is not installed. Unable to send notifications.")
+        logger.warning("dbus-fast is not installed. Unable to send notifications.")
         return -1
 
     id_ = randint(10, 1000) if id_ is None else id_
@@ -426,9 +427,10 @@ async def _send_dbus_message(
     body: Any,
     negotiate_unix_fd: bool = False,
     bus: MessageBus | None = None,
+    preserve: bool = False,
 ) -> tuple[MessageBus | None, Message | None]:
     """
-    Private method to send messages to dbus via dbus_next.
+    Private method to send messages to dbus via dbus_fast.
 
     An existing bus connection can be passed, if left empty, a new
     bus connection will be created.
@@ -452,7 +454,7 @@ async def _send_dbus_message(
     if isinstance(body, str):
         body = [body]
 
-    # Ignore types here: dbus-next has default values of `None` for certain
+    # Ignore types here: dbus-fast has default values of `None` for certain
     # parameters but the signature is `str` so passing `None` results in an
     # error in mypy.
     msg = await bus.call(
@@ -470,7 +472,8 @@ async def _send_dbus_message(
     # Keep details of bus connections so we can close them on exit
     # dbus_bus_connetions is a set so we don't need to worry about
     # duplicates
-    dbus_bus_connections.add(bus)
+    if not preserve:
+        dbus_bus_connections.add(bus)
 
     return bus, msg
 
@@ -483,10 +486,12 @@ async def add_signal_receiver(
     bus_name: str | None = None,
     path: str | None = None,
     check_service: bool = False,
+    use_bus: MessageBus | None = None,
+    preserve: bool = False,
 ) -> bool:
     """
     Helper function which aims to recreate python-dbus's add_signal_receiver
-    method in dbus_next with asyncio calls.
+    method in dbus_fast with asyncio calls.
 
     If check_service is `True` the method will raise a wanrning and return False
     if the service is not visible on the bus. If the `bus_name` is None, no
@@ -495,7 +500,7 @@ async def add_signal_receiver(
     Returns True if subscription is successful.
     """
     if not has_dbus:
-        logger.warning("dbus-next is not installed. Unable to subscribe to signals")
+        logger.warning("dbus-fast is not installed. Unable to subscribe to signals")
         return False
 
     if bus_name and check_service:
@@ -514,7 +519,7 @@ async def add_signal_receiver(
     }
 
     rule = "type='signal',"
-    rule += ",".join("{}='{}'".format(k, v) for k, v in match_args.items() if v)
+    rule += ",".join(f"{k}='{v}'" for k, v in match_args.items() if v)
 
     logger.debug("Adding dbus match rule: %s", rule)
 
@@ -527,13 +532,15 @@ async def add_signal_receiver(
         "AddMatch",
         "s",
         [rule],
+        bus=use_bus,
+        preserve=preserve,
     )
 
     # Check if message sent successfully
     if bus and msg and msg.message_type == MessageType.METHOD_RETURN:
 
         def match_message(msg: Message, match_args: dict[str, str | None]) -> bool:
-            return msg._matches(**{k: v for k, v in match_args.items() if v})
+            return all(getattr(msg, k) == v for k, v in match_args.items() if v)
 
         async def resolve_sender(signal_msg: Message) -> tuple[str, Message]:
             """Looks up a pretty bus name to retrieve the unique name."""
@@ -620,3 +627,18 @@ def remove_dbus_rules() -> None:
         # We need to manually close the socket until https://github.com/altdesktop/python-dbus-next/pull/148
         # gets merged. There's no error on multiple calls to 'close()'.
         bus._sock.close()
+
+
+def reap_zombies() -> None:
+    """
+    A SIGCHLD handler that reaps all zombies until there are no more.
+    """
+    try:
+        # One signal might mean mulitple children have exited. Reap everything
+        # that has exited, until there's nothing left.
+        while True:
+            wait_result = os.waitid(os.P_ALL, 0, os.WEXITED | os.WNOHANG)
+            if wait_result is None:
+                return
+    except ChildProcessError:
+        pass
