@@ -42,7 +42,7 @@ from libqtile.command import interface
 from libqtile.command.base import CommandError, CommandObject, expose_command
 from libqtile.lazy import LazyCall
 from libqtile.log_utils import logger
-from libqtile.utils import create_task
+from libqtile.utils import ColorType, create_task
 
 if TYPE_CHECKING:
     from typing import Any
@@ -146,6 +146,7 @@ class _Widget(CommandObject, configurable.Configurable):
             {},
             "Dict of mouse button press callback functions. Accepts functions and ``lazy`` calls.",
         ),
+        ("hide_crash", False, "Don't display error in bar if widget crashes on startup."),
     ]
 
     def __init__(self, length, **config):
@@ -217,7 +218,6 @@ class _Widget(CommandObject, configurable.Configurable):
     def timer_setup(self):
         """This is called exactly once, after the widget has been configured
         and timers are available to be set up."""
-        pass
 
     def _configure(self, qtile, bar):
         self._test_orientation_compatibility(bar.horizontal)
@@ -245,9 +245,8 @@ class _Widget(CommandObject, configurable.Configurable):
 
         Widgets that need to use asyncio coroutines after this point may
         wish to initialise the relevant code (e.g. connections to dbus
-        using dbus_next) here.
+        using dbus_fast) here.
         """
-        pass
 
     def finalize(self):
         for future in self._futures:
@@ -282,13 +281,13 @@ class _Widget(CommandObject, configurable.Configurable):
         self.mouse_callbacks = defaults
 
     def button_press(self, x, y, button):
-        name = "Button{0}".format(button)
+        name = f"Button{button}"
         if name in self.mouse_callbacks:
             cmd = self.mouse_callbacks[name]
             if isinstance(cmd, LazyCall):
                 if cmd.check(self.qtile):
                     status, val = self.qtile.server.call(
-                        (cmd.selectors, cmd.name, cmd.args, cmd.kwargs)
+                        (cmd.selectors, cmd.name, cmd.args, cmd.kwargs, False)
                     )
                     if status in (interface.ERROR, interface.EXCEPTION):
                         logger.error("Mouse callback command error %s: %s", cmd.name, val)
@@ -304,7 +303,7 @@ class _Widget(CommandObject, configurable.Configurable):
         """
         w = q.widgets_map.get(name)
         if not w:
-            raise CommandError("No such widget: %s" % name)
+            raise CommandError(f"No such widget: {name}")
         return w
 
     def _items(self, name: str) -> ItemT:
@@ -400,7 +399,7 @@ class _Widget(CommandObject, configurable.Configurable):
         return Mirror(self, background=self.background)
 
     def clone(self):
-        return copy.copy(self)
+        return copy.deepcopy(self)
 
     def mouse_enter(self, x, y):
         pass
@@ -446,9 +445,6 @@ class _Widget(CommandObject, configurable.Configurable):
                 del self._old_draw
 
 
-UNSPECIFIED = bar.Obj("UNSPECIFIED")
-
-
 class _TextBox(_Widget):
     """
     Base class for widgets that are just boxes containing text.
@@ -465,10 +461,9 @@ class _TextBox(_Widget):
         (
             "fmt",
             "{}",
-            "To format the string returned by the widget. For example, if the clock widget \
-             returns '08:46' we can do fmt='time {}' do print 'time 08:46' on the widget. \
-             To format the individual strings like hour and minutes use the format paramater \
-             of the widget (if it has one)",
+            "Format to apply to the string returned by the widget. Main purpose: applying markup. "
+            "For a widget that returns ``foo``, using ``fmt='<i>{}</i>'`` would give you ``<i>foo</i>``. "
+            "To control what the widget outputs in the first place, use the ``format`` paramater of the widget (if it has one).",
         ),
         ("max_chars", 0, "Maximum number of characters to display in widget."),
         (
@@ -500,6 +495,7 @@ class _TextBox(_Widget):
             "When ``scroll=True`` the ``width`` parameter is a maximum width and, when text is shorter than this, the widget will resize. "
             "Setting ``scroll_fixed_width=True`` will force the widget to have a fixed width, regardless of the size of the text.",
         ),
+        ("rotate", True, "Rotate text in vertical bar."),
     ]  # type: list[tuple[str, Any, str]]
 
     def __init__(self, text=" ", width=bar.CALCULATED, **config):
@@ -524,11 +520,18 @@ class _TextBox(_Widget):
             value = value[: self.max_chars] + "…"
         self._text = value
         if self.layout:
+            # Reset the layout width
+            # Reason is because, if we've manually set the layout width,
+            # adding longer text will result in wrapping which increases the height of the layout.
+            del self.layout.width
             self.layout.text = self.formatted_text
             if self.scroll:
                 self.check_width()
                 self.reset_scroll()
 
+            elif not self.bar.horizontal and not self.rotate:
+                self.layout.width = self.bar.width - 2 * self.actual_padding
+
     @property
     def formatted_text(self):
         return self.fmt.format(self._text)
@@ -583,24 +586,34 @@ class _TextBox(_Widget):
             markup=self.markup,
         )
         if not isinstance(self._scroll_width, int) and self.scroll:
-            logger.warning("%s: You must specify a width when enabling scrolling.", self.name)
-            self.scroll = False
+            if not self.bar.horizontal and not self.rotate:
+                self._scroll_width = self.bar.width
+                self.scroll_fixed_width = self.bar.width
+            else:
+                logger.warning("%s: You must specify a width when enabling scrolling.", self.name)
+                self.scroll = False
 
         if self.scroll:
             self.check_width()
 
+        elif not self.bar.horizontal and not self.rotate:
+            self.layout.width = self.bar.width - 2 * self.actual_padding
+
     def check_width(self):
         """
         Check whether the widget needs to have calculated or fixed width
         and whether the text should be scrolled.
         """
         if self.layout.width > self._scroll_width:
-            self.length_type = bar.STATIC
-            self.length = self._scroll_width
+            if self.bar.horizontal or self.rotate:
+                self.length_type = bar.STATIC
+                self.length = self._scroll_width
             self._is_scrolling = True
             self._should_scroll = True
         else:
-            if self.scroll_fixed_width:
+            if not self.bar.horizontal and not self.rotate:
+                self.layout.width = self.scroll_fixed_width
+            elif self.scroll_fixed_width:
                 self.length_type = bar.STATIC
                 self.length = self._scroll_width
             else:
@@ -612,7 +625,10 @@ class _TextBox(_Widget):
             if self.bar.horizontal:
                 return min(self.layout.width, self.bar.width) + self.actual_padding * 2
             else:
-                return min(self.layout.width, self.bar.height) + self.actual_padding * 2
+                if self.rotate:
+                    return min(self.layout.width, self.bar.height) + self.actual_padding * 2
+                else:
+                    return self.layout.height + self.actual_padding * 2
         else:
             return 0
 
@@ -630,7 +646,7 @@ class _TextBox(_Widget):
         # size = self.bar.height if self.bar.horizontal else self.bar.width
         self.drawer.ctx.save()
 
-        if not self.bar.horizontal:
+        if not self.bar.horizontal and self.rotate:
             # Left bar reads bottom to top
             if self.bar.screen.left is self.bar:
                 self.drawer.ctx.rotate(-90 * math.pi / 180.0)
@@ -652,7 +668,13 @@ class _TextBox(_Widget):
             )
             self.drawer.ctx.clip()
 
-        size = self.bar.height if self.bar.horizontal else self.bar.width
+        if self.bar.horizontal:
+            size = self.bar.height
+        else:
+            if self.rotate:
+                size = self.bar.width
+            else:
+                size = self.layout.height + self.actual_padding * 2
 
         self.layout.draw(
             (self.actual_padding or 0) - self._scroll_offset,
@@ -718,16 +740,21 @@ class _TextBox(_Widget):
         self.update("")
 
     @expose_command()
-    def set_font(self, font=UNSPECIFIED, fontsize=UNSPECIFIED, fontshadow=UNSPECIFIED):
+    def set_font(
+        self,
+        font: str | None = None,
+        fontsize: int = 0,
+        fontshadow: ColorType = "",
+    ):
         """
         Change the font used by this widget. If font is None, the current
         font is used.
         """
-        if font is not UNSPECIFIED:
+        if font is not None:
             self.font = font
-        if fontsize is not UNSPECIFIED:
+        if fontsize != 0:
             self.fontsize = fontsize
-        if fontshadow is not UNSPECIFIED:
+        if fontshadow != "":
             self.fontshadow = fontshadow
         self.bar.draw()
 
@@ -757,7 +784,7 @@ class _TextBox(_Widget):
 
         # If our width hasn't changed, we just draw ourselves. Otherwise,
         # we draw the whole bar.
-        if self.layout.width == old_width:
+        if self.layout.width == old_width and (self.bar.horizontal or self.rotate):
             self.draw()
         else:
             self.bar.draw()
@@ -769,7 +796,7 @@ class InLoopPollText(_TextBox):
     ThreadPoolText instead.
 
     ('fast' here means that this runs /in/ the event loop, so don't block! If
-    you want to run something nontrivial, use ThreadedPollWidget.)"""
+    you want to run something nontrivial, use ThreadPoolText.)"""
 
     defaults = [
         (
@@ -794,14 +821,6 @@ class InLoopPollText(_TextBox):
             self.timeout_add(update_interval, self.timer_setup)
         # If update_interval is False, we won't re-call
 
-    def _configure(self, qtile, bar):
-        should_tick = self.configured
-        _TextBox._configure(self, qtile, bar)
-
-        # Update when we are being re-configured.
-        if should_tick:
-            self.tick()
-
     def button_press(self, x, y, button):
         self.tick()
         _TextBox.button_press(self, x, y, button)
@@ -834,7 +853,7 @@ class ThreadPoolText(_TextBox):
         ),
     ]  # type: list[tuple[str, Any, str]]
 
-    def __init__(self, text, **config):
+    def __init__(self, text="N/A", **config):
         super().__init__(text, **config)
         self.add_defaults(ThreadPoolText.defaults)
 
@@ -938,6 +957,8 @@ class Mirror(_Widget):
         self.reflects = reflection
         self._length = 0
         self.length_type = self.reflects.length_type
+        if self.length_type is bar.STATIC:
+            self._length = self.reflects._length
 
     def _configure(self, qtile, bar):
         _Widget._configure(self, qtile, bar)
@@ -960,7 +981,9 @@ class Mirror(_Widget):
         self._length = value
 
     def draw(self):
-        self.drawer.clear(self.reflects.background or self.bar.background)
+        if self.length <= 0:
+            return
+        self.drawer.clear_rect()
         self.reflects.drawer.paint_to(self.drawer)
         self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.width)
 
